require 'spec_helper'

describe Grape::Formatter::Paginater do

  subject do
    Class.new(Grape::API)
  end

  def app
    subject
  end

  describe '#header' do
    it 'is callable from within a block' do
      subject.get('/hey') do
        header 'X-Awesome', 'true'
        "Awesome"
      end

      get '/hey'
      last_response.headers['X-Awesome'].should == 'true'
    end
  end

  describe '#headers' do
    before do
      subject.get('/headers') do
        headers.to_json
      end
    end
    it 'includes request headers' do
      get '/headers'
      JSON.parse(last_response.body).should == {
        "Host" => "example.org",
        "Cookie" => ""
      }
    end
    it 'includes additional request headers' do
      get '/headers', nil, { "HTTP_X_GRAPE_CLIENT" => "1" }
      JSON.parse(last_response.body)["X-Grape-Client"].should == "1"
    end
  end

  context "without paginater" do

    before :each do
      stuff = {:stuff => 'foo bar'}
    end

    it "sets itself in the env upon call" do
      subject.get('/') do
        'Hello world.'
      end
      get '/'
      last_request.env['api.endpoint'].should be_kind_of(Grape::Endpoint)
    end
  
    it "should work without paginater" do
      subject.get('/') do
        'foo bar!'
      end
      get '/'
      last_response.body.should eq 'foo bar!'
    end
  
    it "should not raise error about stuff" do
      subject.format :json
      subject.get '/home' do
        stuff = {:stuff => 'foo bar'}
      end
      lambda do
        get '/home'
      end.should_not raise_error
      get '/home'
      last_response.body.should eq "{\"stuff\":\"foo bar\"}"
      last_response.body.should include MultiJson.dump stuff
    end

  end

  context "with the route_option :paginater => :simple" do

    before :each do
      subject.format :json
      subject.formatter :json, Grape::Formatter::Paginater
      stuff = {:stuff => 'foo bar'}
    end

    it "should respond with proper content-type" do
      subject.get '/home', :paginater => :simple do
        stuff = {:stuff => 'foo bar'}
      end
      get('/home?page=1')
      last_response.headers['Content-Type'].should == 'application/json'
    end

    it 'rescues a specific error' do
      subject.rescue_from Grape::Formatter::Paginater::Constants::PaginaterError do |e|
        rack_response("rescued from #{e.class.name}", 500)
      end
      subject.get '/home', :paginater => :simple do
        raise Grape::Formatter::Paginater::Constants::PaginaterError unless params[:page]
      end
      subject.get '/foo' do
        raise 'foo_exception'
      end
      get '/home'
      last_response.status.should eql 500
      last_response.body.should == 'rescued from Grape::Formatter::Paginater::Constants::PaginaterError'
      lambda do
        get '/foo'
      end.should raise_error
    end

    it 'should understand a missing page situation' do
      subject.get '/home', :paginater => :simple do
        params[:page]
        stuff = {:stuff => 'foo bar'}
      end
      lambda do 
        get '/home'
      end.should_not raise_error
      last_response.status.should eql 500
      last_response.body.should eq '{"error":"missing param"}'
    end

    it 'defaults the error formatter to format' do
      subject.format :json
      subject.rescue_from :all
      subject.content_type :json, "application/json"
      subject.content_type :foo, "text/foo"
      subject.get '/exception' do
        raise "rain!"
      end
      get '/exception.json'
      last_response.body.should == '{"error":"rain!"}'
      get '/exception.foo'
      last_response.body.should == '{"error":"rain!"}'
    end

    it 'should work with the params requested' do
      subject.format :json
      subject.rescue_from :all
      subject.get '/stuff', :paginater => :simple do
        params[:page]
        stuff = {:stuff => 'foo bar'}
      end
      get '/stuff?page=42'
      last_response.body.should eq '{"error":"invalid page 42"}'
      last_response.status.should eq 500
    end

  end

  context "dealing with something that ... count" do

    before :each do
      subject.format :json
      subject.formatter :json, Grape::Formatter::Paginater
      subject.get '/foo', :paginater => :simple do
        numbers(42)
      end
    end
    it 'consider Array objects as paginable' do
      get('/foo?page=11')
      last_response.status.should eq 200
      last_response.header.should include 'Link'
    end
    it 'paginate the body' do
      get('/foo?page=11')
      last_response.status.should eq 200
      last_response.body.should include '10'
    end
    it 'paginate the body and understand size' do
      get('/foo?page=3&size=3')
      last_response.status.should eq 200
      last_response.body.should_not include '5'
      last_response.body.should include '6'
      last_response.body.should include '8'
      last_response.body.should_not include '9'
      last_response.body.should include 6.upto(8).to_a.to_s.gsub(/\s/,'')
    end
    it 'set headers for pagination' do
      get('/foo?page=3&size=3')
      last_response.status.should eq 200
      last_response.header.should include 'Link'
      last_response.header['Link'].should include 'page=1'
      last_response.header['Link'].should include 'next'
      last_response.header['Link'].should include 'page=4'
      last_response.header['Link'].should include 'prev'
      last_response.header['Link'].should include 'page=2'
    end
    it 'do not mess with last pages' do
      get('/foo?page=5&size=10')
      last_response.status.should eq 200
      last_response.header.should include 'Link'
      last_response.header['Link'].should include 'prev'
      last_response.header['Link'].should_not include 'next'
      last_response.body.should_not include '39'
      last_response.body.should include '40'
      last_response.body.should include '42'
      last_response.body.should_not include '43'
      last_response.body.should include 41.upto(42).to_a.to_s.gsub(/[\[\s\]]/,'')
    end
    it 'do not mess with foo pages' do
      get('/foo?page=42&size=42')
      last_response.status.should eq 500
    end

  end

  context "dealing with something that ... count but :txt" do

    before :each do
      subject.format :txt
      subject.formatter :txt, Grape::Formatter::Paginater
      subject.get '/foo', :paginater => :simple do
        alphabet
      end
    end
    it 'consider Array objects as paginable' do
      get('/foo?page=11')
      last_response.status.should eq 200
      last_response.header['Link'].should include 'first'
    end

    it 'understand and return a txt' do
      get('/foo?page=1&size=10')
      last_response.body.should include "Alpha"
      last_response.body.should_not include "Kilo"
      last_response.header['Link'].should include "page=1"
      last_response.body.split(',').count.should eq 10
    end

  end

  context "set pagination headers" do

    before :each do
      subject.format :json
      subject.formatter :json, Grape::Formatter::Paginater
      subject.get '/foo', :paginater => :simple do
        items(9)
      end
    end
    it 'understand the first page' do
      get('/foo?page=1')
      last_response.status.should eq 200
      last_response.header['Link'].should include 'page=1'
      last_response.header['Link'].should include 'next'
      last_response.header['Link'].should_not include 'prev'
    end
    it 'understand the last page' do
      get('/foo?page=10')
      last_response.status.should eq 200
      last_response.header['Link'].should include 'prev'
      last_response.header['Link'].should include 'page=9'
      last_response.header['Link'].should_not include 'next'
    end
    it 'understand the last page with size' do
      get('/foo?page=5&size=2')
      last_response.status.should eq 200
      last_response.header['Link'].should include 'prev'
      last_response.header.should_not include 'next'
      last_response.body.split(',').count.should eq 2
      get('/foo?page=4&size=3')
      last_response.status.should eq 200
      last_response.header['Link'].should include 'prev'
      last_response.header['Link'].should include 'page=3'
      last_response.header.should_not include 'next'
      last_response.body.split(',').count.should eq 1
    end
    it 'paginate the body' do
      get('/foo?page=1')
      last_response.status.should eq 200
      last_response.body.should eq '["item 000"]'
      get('/foo?page=10')
      last_response.body.should eq '["item 009"]'
      get('/foo?page=11')
      last_response.status.should eq 500
      get('/foo?page=3&size=3')
      last_response.body.should eq '["item 006","item 007","item 008"]'
    end

  end

  context "with the wrapped object" do

    before :each do
      subject.format :json
      subject.formatter :json, Grape::Formatter::Paginater
      subject.get '/foobar', :paginater => :wrapper do
        items(42)
      end
    end
    it "get something paginated" do
      get('/foobar?page=1')
      last_response.status.should eq 200
      # deprecating....
      last_response.header['Link'].should include 'page', 'next'
      last_response.header.should_not include 'prev'
      last_response.body.should include '["item 000"]'
      last_response.body.should eq "{\"content\":[\"item 000\"],\"page\":1,\"size\":1,\"links\":null}"
      get('/foobar?page=3&size=3')
      last_response.body.should eq "{\"content\":[\"item 006\",\"item 007\",\"item 008\"],\"page\":3,\"size\":3,\"links\":null}"
    end

  end

  context "put something in the cookies" do
    
    before :each do
      subject.format :json
      subject.formatter :json, Grape::Formatter::Paginater
      subject.get '/foobar', :paginater => :simple do
        items(42)
      end
    end
    
    it "count total_pages and original_count" do
      get('/foobar?page=7&size=3')
      last_response.header['Set-Cookie'].should include "7/14/43"
    end

  end

end # Grape::Formatter::Paginater
