﻿# spec/paginater/paginater_with_kaminari_spec.rb

require 'spec_helper'
require 'support/active_record_support'

if defined? ActiveRecord
  
  describe Grape::Formatter::Paginater do
    
    subject do
      Class.new(Grape::API)
    end

    def app
      subject
    end

    shared_examples_for 'the first page' do
      it { should have(25).users }
      its('first.name') { should == 'user001' }
    end

    shared_examples_for 'blank page' do
      it { should have(0).users }
    end

    before :each do
      0.upto(999) {|i| User.create! :name => "user#{'%03d' % i}", :age => (i / 10)}
    end

    after :each do
      User.destroy_all
    end

    context "with the kaminari amazing gem" do

      before :each do
        subject.format :json
        subject.formatter :json, Grape::Formatter::Paginater
        subject.get '/users', :paginater => :kaminari do
          User.where("age > 0")
        end
      end

      it "get something paginated" do
        get('/users?page=42&size=10')
        last_response.status.should eq 200
        last_response.header['Link'].should include 'first', 'next'
        last_response.header.should_not include 'prev'
        last_response.body.should include '{"user":{"age":42,"id":425,"name":"user424"}}'
        last_response.header['Link'].should include "{:url=>\"/users?page=1&size=10\", :rel=>\"first\"}"
        last_response.header['Link'].should include "{:url=>\"/users?page=43&size=10\", :rel=>\"next\"}"
        last_response.header['Link'].should include "{:url=>\"/users?page=41&size=10\", :rel=>\"prev\"}"
      end
    
    end

    context "some features with kaminari" do

      # let(:all_user_count) { User.all.count }
      
      before :each do
        subject.format :json
        subject.formatter :json, Grape::Formatter::Paginater
        subject.get '/users', :paginater => :kaminari do
          User.where("age > -1")
        end
      end

      it "get something paginated with body, headers and cookies" do
        get('/users?page=1&size=10')
        last_response.status.should eq 200
        last_response.body.scan(/\"user\"/).flatten.count.should == 10
        last_response.header['Set-Cookie'].should include "1/100/1000" #page/pages/count
        last_response.header['Link'].should include '/users?page=2&size=10'
        last_response.header['Link'].should_not include 'prev'
      end

      it "use the default size of 1" do
        get('/users?page=1')
        last_response.status.should eq 200
        last_response.body.scan(/\"user\"/).flatten.count.should == 1
        last_response.header['Set-Cookie'].should include "1/1000/1000"
        last_response.header['Link'].should include '/users?page=2'
      end

      it "manage an exceeding size as .all" do
        get('/users?page=1&size=9999')
        last_response.status.should eq 200
        last_response.body.scan(/\"user\"/).flatten.count.should == 1000
        last_response.header['Set-Cookie'].should include "1/1/1000"
        last_response.header['Link'].should_not include '/users?page=2'
      end
      
    end

    context "with kaminari and an array (PaginatableArray)" do

      before :each do
        subject.format :json
        subject.formatter :json, Grape::Formatter::Paginater
        subject.get '/foobar', :paginater => :kaminari do
          items(99)
        end
      end
      it "get something paginated (with default size)" do
        get('/foobar?page=1')
        last_response.status.should eq 200
      end
      it "get something paginated" do
        get('/foobar?page=1&size=10')
        last_response.status.should eq 200
        last_response.header['Link'].should include "{:url=>\"/foobar?page=1&size=10\", :rel=>\"first\"}" 
        last_response.header['Link'].should include "{:url=>\"/foobar?page=2&size=10\", :rel=>\"next\"}"
        last_response.body.should include "item 009"
        last_response.body.should_not include "item 010"
      end
      it "get something paginated with default size = 1" do
        get('/foobar?page=100')
        last_response.body.scan(/"(.*?)"/).flatten.count.should == 1
        last_response.header['Link'].should_not include "next"
      end
      it "get something paginated with the size param" do
        get('/foobar?page=10&size=10')
        last_response.body.scan(/"(.*?)"/).flatten.count.should == 10
        last_response.header['Link'].should_not include "next"
      end
      it "understand a non existing page" do
        get('/foobar?page=11&size=10')
        last_response.body.scan(/"(.*?)"/).flatten.count.should == 0
        last_response.body.should == '[]'
        # FIXME?
        # p last_response.header['Link']
        # p last_response.header['Set-Cookie']
      end
    
    end

    context "with kaminari and an hash" do

      before :each do
        subject.format :json
        subject.formatter :json, Grape::Formatter::Paginater
        subject.get '/foobar', :paginater => :kaminari do
          [{:foo => 'bar'},{:bar => 'foo'}]
        end
      end
      it "get something paginated" do
        get('/foobar?page=1')
        last_response.status.should eq 200
        last_response.body.should include "[{\"foo\":\"bar\"}]"
        last_response.header['Link'].should include "{:url=>\"/foobar?page=1\", :rel=>\"first\"}"
      end
      it "discern errors and override size with 1 if necessary" do
        get('/foobar?page=1&size=0')
        last_response.status.should eq 200
        last_response.body.should == "[{\"foo\":\"bar\"}]"
      end
    
    end

  end # Grape::Formatter::Paginater

end

